==> mimizuku-0.1.11.tar.gz: mimizuku-0.1.11/mimizuku/model.py <==
import json
import os
import re

import joblib
import pandas as pd
from scipy.sparse import hstack
from sklearn.ensemble import IsolationForest
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.preprocessing import LabelEncoder

from .utils import anonymize_path, safe_transform


class Mimizuku:
    def __init__(
        self, n_estimators=1000, random_state=42, max_samples=0.8, contamination=0.01
    ):
        self.model = IsolationForest(
            n_estimators=n_estimators,
            random_state=random_state,
            max_samples=max_samples,
            contamination=contamination,
        )
        self.tfidf_vectorizer_path = TfidfVectorizer(stop_words=None)
        self.le_hostname = LabelEncoder()

    def load_and_preprocess(self, data, fit=False, keep_original=False):
        alerts = []
        if isinstance(data, str):
            with open(data, encoding="utf-8", errors="replace") as json_file:
                for line in json_file:
                    try:
                        alert = json.loads(line)
                        if (
                            # change file messages
                            re.match(r"55[0-9]", str(alert["rule"]["id"]))
                        ):
                            alerts.append(alert)
                    except json.JSONDecodeError as e:
                        print(f"Error decoding JSON: {e}")
        elif isinstance(data, pd.DataFrame):
            alerts = data.to_dict("records")
        else:
            raise ValueError("Input data must be a filepath or a DataFrame")

        processed_data = []
        for alert in alerts:
            if "syscheck" not in alert:
                continue
            path = alert.get("syscheck", {}).get("path")

            row = {
                "hostname": re.sub("[0-9]", "*", alert.get("agent", {}).get("name")),
                "path": anonymize_path(path).replace("/", " "),
                "id": alert.get("rule", {}).get("id"),
            }

            if keep_original:
                row.update(
                    {
                        "original_hostname": alert.get("agent", {}).get("name"),
                        "original_path": path,
                    }
                )

            processed_data.append(row)

        df = pd.DataFrame(processed_data)

        if len(df) == 0:
            raise ValueError("No alerts were found in the input data")

        if fit:
            df["hostname_encoded"] = self.le_hostname.fit_transform(df["hostname"])
            path_tfidf = self.tfidf_vectorizer_path.fit_transform(df["path"])
        else:
            df["hostname_encoded"] = safe_transform(self.le_hostname, df["hostname"])
            path_tfidf = self.tfidf_vectorizer_path.transform(df["path"])

        X = hstack([path_tfidf, df["hostname_encoded"].values[:, None]])
        return X, df

    def fit(self, data):
        try:
            X_train, _ = self.load_and_preprocess(data, fit=True)
            self.model.fit(X_train)
        except ValueError as e:
            if "No alerts were found" in str(e):
                pass
            else:
                raise e

    def predict(self, data):
        try:
            X_test, df_test = self.load_and_preprocess(
                data, fit=False, keep_original=True
            )
            anomalies = self.model.predict(X_test)
            anomalies_df = df_test[anomalies == -1]
            return anomalies_df[
                [
                    "original_hostname",
                    "original_path",
                ]
            ]
        except ValueError as e:
            if "No alerts were found" in str(e):
                pass
            else:
                raise e

    def save_model(self, model_path):
        joblib.dump(
            {
                "model": self.model,
                "tfidf_vectorizer_path": self.tfidf_vectorizer_path,
                "le_hostname": self.le_hostname,
            },
            model_path,
        )

    @staticmethod
    def load_model(model_path):
        saved_objects = joblib.load(model_path)
        mimizuku = Mimizuku()
        mimizuku.model = saved_objects["model"]
        mimizuku.tfidf_vectorizer_path = saved_objects["tfidf_vectorizer_path"]
        mimizuku.le_hostname = saved_objects["le_hostname"]
        return mimizuku

==> mimizuku-0.2.0.tar.gz: mimizuku-0.2.0/mimizuku/model.py <==
import json
import re

import joblib
import numpy as np
import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.neighbors import LocalOutlierFactor


class Mimizuku:
    def __init__(self, n_neighbors=15, contamination=0.05):
        self.model = LocalOutlierFactor(
            n_neighbors=n_neighbors, contamination=contamination, novelty=True
        )
        self.tfidf_vectorizer = TfidfVectorizer()

    def load_and_preprocess(self, data, fit=False, keep_original=False):
        alerts = []
        if isinstance(data, str):
            with open(data, encoding="utf-8", errors="replace") as json_file:
                for line in json_file:
                    try:
                        alert = json.loads(line)
                        if "syscheck" in alert and re.match(
                            r"55[0-9]", str(alert["rule"]["id"])
                        ):
                            alerts.append(alert)
                    except json.JSONDecodeError as e:
                        print(f"Error decoding JSON: {e}")
        elif isinstance(data, pd.DataFrame):
            alerts = data.to_dict("records")
        else:
            raise ValueError("Input data must be a filepath or a DataFrame")

        processed_data = []
        for alert in alerts:
            syscheck = alert.get("syscheck", {})
            path = syscheck.get("path")
            size_after = syscheck.get("size_after", 0)
            uid_after = syscheck.get("uid_after", 0)
            event = syscheck.get("event", "")

            row = {
                "hostname": alert.get("agent", {}).get("name"),
                "path": path,
                "size_after": size_after,
                "uid_after": uid_after,
                "event": event,
            }

            if keep_original:
                row.update(
                    {
                        "original_hostname": alert.get("agent", {}).get("name"),
                        "original_path": path,
                    }
                )

            processed_data.append(row)

        df = pd.DataFrame(processed_data)

        if len(df) == 0:
            raise ValueError("No alerts were found in the input data")

        combined_text = df["hostname"] + " " + df["path"] + " " + df["event"]
        if fit:
            tfidf_features = self.tfidf_vectorizer.fit_transform(combined_text)
        else:
            tfidf_features = self.tfidf_vectorizer.transform(combined_text)

        size_uid_features = df[["size_after", "uid_after"]].values

        X = np.hstack([tfidf_features.toarray(), size_uid_features])
        return X, df

    def fit(self, data):
        try:
            X_train, _ = self.load_and_preprocess(data, fit=True)
            self.model.fit(X_train)
        except ValueError as e:
            if "No alerts were found" in str(e):
                pass
            else:
                raise e

    def predict(self, data):
        try:
            X_test, df_test = self.load_and_preprocess(
                data, fit=False, keep_original=True
            )
            anomalies = self.model.predict(X_test)
            anomalies_df = df_test[anomalies == -1]
            return anomalies_df[
                [
                    "original_hostname",
                    "original_path",
                ]
            ]
        except ValueError as e:
            if "No alerts were found" in str(e):
                pass
            else:
                raise e

    def save_model(self, model_path):
        joblib.dump(
            {
                "model": self.model,
                "tfidf_vectorizer": self.tfidf_vectorizer,
            },
            model_path,
        )

    @staticmethod
    def load_model(model_path):
        saved_objects = joblib.load(model_path)
        mimizuku = Mimizuku()
        mimizuku.model = saved_objects["model"]
        mimizuku.tfidf_vectorizer = saved_objects["tfidf_vectorizer"]
        return mimizuku
